@@ -1,10 +1,8 @@
-# Copyright (c) 2008-2014 Simplistix Ltd
-# See license.txt for license details.
-
 from collections import defaultdict
 import atexit
 import logging
 import warnings
+from pprint import pformat
 
 from testfixtures.comparison import compare
 from testfixtures.utils import wrap
@@ -30,7 +28,7 @@ class LogCapture(logging.Handler):
     :param attributes:
 
       The sequence of attribute names to return for each record or a callable
-      that extracts a row from a record..
+      that extracts a row from a record.
 
       If a sequence of attribute names, those attributes will be taken from the
       :class:`~logging.LogRecord`. If an attribute is callable, the value
@@ -88,7 +86,8 @@ class LogCapture(logging.Handler):
         framework for the named loggers.
 
         This will remove any existing handlers for those loggers and
-        drop their level to 1 in order to capture all logging.
+        drop their level to that specified on this :class:`LogCapture` in order
+        to capture all logging.
         """
         for name in self.names:
             logger = logging.getLogger(name)
@@ -138,15 +137,26 @@ class LogCapture(logging.Handler):
             yield value
 
     def actual(self):
+        """
+        The sequence of actual records logged, having had their attributes
+        extracted as specified by the ``attributes`` parameter to the
+        :class:`LogCapture` constructor.
+
+        This can be useful for making more complex assertions about logged
+        records. The actual records logged can also be inspected by using the
+        :attr:`records` attribute.
+        """
+        actual = []
         for r in self.records:
             if callable(self.attributes):
-                yield self.attributes(r)
+                actual.append(self.attributes(r))
             else:
                 result = tuple(self._actual_row(r))
                 if len(result) == 1:
-                    yield result[0]
+                    actual.append(result[0])
                 else:
-                    yield result
+                    actual.append(result)
+        return actual
 
     def __str__(self):
         if not self.records:
@@ -159,16 +169,79 @@ class LogCapture(logging.Handler):
         entries provided and raise an :class:`AssertionError` if they
         do not match.
 
-        :param expected: A sequence of 3-tuples containing the
-                         expected log entries. Each tuple should be of
-                         the form (logger_name, string_level, message)
+        :param expected:
+
+          A sequence of entries of the structure specified by the ``attributes``
+          passed to the constructor.
         """
         return compare(
             expected,
-            actual=tuple(self.actual()),
+            actual=self.actual(),
             recursive=self.recursive_check
             )
 
+    def check_present(self, *expected, **kw):
+        """
+        This will check if the captured entries contain all of the expected
+        entries provided and raise an :class:`AssertionError` if not.
+        This will ignore entries that have been captured but that do not
+        match those in ``expected``.
+
+        :param expected:
+
+          A sequence of entries of the structure specified by the ``attributes``
+          passed to the constructor.
+
+        :param order_matters:
+
+          A keyword-only parameter that controls whether the order of the
+          captured entries is required to match those of the expected entries.
+          Defaults to ``True``.
+        """
+        order_matters = kw.pop('order_matters', True)
+        assert not kw, 'order_matters is the only keyword parameter'
+        actual = self.actual()
+        if order_matters:
+            matched_indices = [0]
+            matched = []
+            for entry in expected:
+                try:
+                    index = actual.index(entry, matched_indices[-1])
+                except ValueError:
+                    if len(matched_indices) > 1:
+                        matched_indices.pop()
+                        matched.pop()
+                    break
+                else:
+                    matched_indices.append(index+1)
+                    matched.append(entry)
+            else:
+                return
+
+            compare(expected,
+                    actual=matched+actual[matched_indices[-1]:],
+                    recursive=self.recursive_check)
+        else:
+            expected = list(expected)
+            matched = []
+            unmatched = []
+            for entry in actual:
+                try:
+                    index = expected.index(entry)
+                except ValueError:
+                    unmatched.append(entry)
+                else:
+                    matched.append(expected.pop(index))
+                if not expected:
+                    break
+            if expected:
+                raise AssertionError((
+                    'entries not as expected:\n\n'
+                    'expected and found:\n%s\n\n'
+                    'expected but not found:\n%s\n\n'
+                    'other entries:\n%s'
+                ) % (pformat(matched), pformat(expected), pformat(unmatched)))
+
     def __enter__(self):
         return self
 
@@ -180,6 +253,7 @@ class LogCaptureForDecorator(LogCapture):
 
     def install(self):
         LogCapture.install(self)
+        self.clear()
         return self
 
 
