@@ -1,10 +1,14 @@
-# Copyright (c) 2008-2013 Simplistix Ltd
-# See license.txt for license details.
-
-from testfixtures import log_capture, compare, Comparison as C, LogCapture
-from unittest import TestCase
+from __future__ import absolute_import
 
 from logging import getLogger
+from unittest import TestCase
+
+from testfixtures.shouldraise import ShouldAssert
+from .mock import patch
+
+from testfixtures import (
+    log_capture, compare, Comparison as C, LogCapture, ShouldRaise
+)
 
 root = getLogger()
 one = getLogger('one')
@@ -156,9 +160,84 @@ class TestLog_Capture(TestCase):
             dict(foo='bar', baz='bob'),
         )
 
+    def test_callable_instead_of_attribute(self):
+        def extract_msg(record):
+            return {k: v for (k, v) in record.msg.items()
+                    if k != 'baz'}
+        with LogCapture(attributes=extract_msg) as log:
+            getLogger().info(dict(foo='bar', baz='bob'))
+        log.check(
+            dict(foo='bar'),
+        )
+
     def test_msg_is_none(self):
         with LogCapture(attributes=('msg', 'foo')) as log:
             getLogger().info(None, extra=dict(foo='bar'))
         log.check(
             (None, 'bar')
         )
+
+    def test_normal_check(self):
+        with LogCapture() as log:
+            getLogger().info('oh hai')
+
+        with ShouldAssert(
+            "sequence not as expected:\n\n"
+            "same:\n"
+            "()\n\n"
+            "expected:\n"
+            "(('root', 'INFO', 'oh noez'),)\n\n"
+            "actual:\n"
+            "(('root', 'INFO', 'oh hai'),)"
+        ):
+            log.check(('root', 'INFO', 'oh noez'))
+
+    def test_recursive_check(self):
+
+        with LogCapture(recursive_check=True) as log:
+            getLogger().info('oh hai')
+
+        with ShouldAssert(
+            "sequence not as expected:\n\n"
+            "same:\n()\n\n"
+            "expected:\n(('root', 'INFO', 'oh noez'),)\n\n"
+            "actual:\n(('root', 'INFO', 'oh hai'),)\n\n"
+            "While comparing [0]: sequence not as expected:\n\n"
+            "same:\n('root', 'INFO')\n\n"
+            "expected:\n"
+            "('oh noez',)\n\n"
+            "actual:\n"
+            "('oh hai',)\n\n"
+            "While comparing [0][2]: 'oh noez' (expected) != 'oh hai' (actual)"
+        ):
+            log.check(('root', 'INFO', 'oh noez'))
+
+    @log_capture()
+    @patch('testfixtures.tests.sample1.SampleClassA')
+    def test_patch_then_log(self, a1, a2):
+        actual = [type(c).__name__ for c in (a1, a2)]
+        compare(actual, expected=['MagicMock', 'LogCaptureForDecorator'])
+
+    @patch('testfixtures.tests.sample1.SampleClassA')
+    @log_capture()
+    def test_log_then_patch(self, a1, a2):
+        actual = [type(c).__name__ for c in (a1, a2)]
+        compare(actual, expected=['LogCaptureForDecorator', 'MagicMock'])
+
+
+class BaseCaptureTest(TestCase):
+    a = 33
+
+    @log_capture()
+    def test_logs_if_a_smaller_than_44(self, logs):
+        logger = getLogger()
+        if self.a < 44:
+            logger.info('{} is smaller than 44'.format(self.a))
+
+        logs.check(
+            ('root', 'INFO', '{} is smaller than 44'.format(self.a)),
+        )
+
+
+class SubclassCaptureTest(BaseCaptureTest):
+    a = 2
